# Generated by Django 2.1.11 on 2019-08-30 16:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0003_control_mutationlog"),
    ]

    operations = [
        migrations.AddField(
            model_name="mutationlog",
            name="client_mutation_label",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name="mutationlog",
            name="status",
            field=models.IntegerField(
                choices=[(0, "Received"), (1, "Error"), (2, "Success")], default=0
            ),
        ),
    ]
