# Generated by Django 3.2.16 on 2024-01-24 11:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0025_mutationlog_json_ext"),
    ]

    operations = [
        migrations.AddField(
            model_name="mutationlog",
            name="autogenerated_code",
            field=models.TextField(blank=True, null=True),
        ),
    ]
