# Generated by Django 3.2.20 on 2023-08-31 10:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0024_alter_usergroup_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="mutationlog",
            name="json_ext",
            field=models.JSONField(blank=True, db_column="JsonExt", null=True),
        ),
    ]
