from core.utils import *  # noqa: F401,F403
from core.datetimes.shared import *  # noqa: F401,F403
from core.custom_lookups import *  # noqa: F401,F403

default_app_config = "core.apps.CoreConfig"

# For IDE support, filled at runtime
datetime = None
